@@ -10,24 +10,60 @@ from typing import Any, Callable, Literal
 
 import pydantic
 import websockets
-from openai.types.beta.realtime.conversation_item import ConversationItem
+from openai.types.beta.realtime.conversation_item import (
+    ConversationItem,
+    ConversationItem as OpenAIConversationItem,
+)
+from openai.types.beta.realtime.conversation_item_content import (
+    ConversationItemContent as OpenAIConversationItemContent,
+)
+from openai.types.beta.realtime.conversation_item_create_event import (
+    ConversationItemCreateEvent as OpenAIConversationItemCreateEvent,
+)
+from openai.types.beta.realtime.conversation_item_retrieve_event import (
+    ConversationItemRetrieveEvent as OpenAIConversationItemRetrieveEvent,
+)
+from openai.types.beta.realtime.conversation_item_truncate_event import (
+    ConversationItemTruncateEvent as OpenAIConversationItemTruncateEvent,
+)
+from openai.types.beta.realtime.input_audio_buffer_append_event import (
+    InputAudioBufferAppendEvent as OpenAIInputAudioBufferAppendEvent,
+)
+from openai.types.beta.realtime.input_audio_buffer_commit_event import (
+    InputAudioBufferCommitEvent as OpenAIInputAudioBufferCommitEvent,
+)
+from openai.types.beta.realtime.realtime_client_event import (
+    RealtimeClientEvent as OpenAIRealtimeClientEvent,
+)
 from openai.types.beta.realtime.realtime_server_event import (
     RealtimeServerEvent as OpenAIRealtimeServerEvent,
 )
 from openai.types.beta.realtime.response_audio_delta_event import ResponseAudioDeltaEvent
+from openai.types.beta.realtime.response_cancel_event import (
+    ResponseCancelEvent as OpenAIResponseCancelEvent,
+)
+from openai.types.beta.realtime.response_create_event import (
+    ResponseCreateEvent as OpenAIResponseCreateEvent,
+)
 from openai.types.beta.realtime.session_update_event import (
     Session as OpenAISessionObject,
     SessionTool as OpenAISessionTool,
+    SessionTracing as OpenAISessionTracing,
+    SessionTracingTracingConfiguration as OpenAISessionTracingConfiguration,
+    SessionUpdateEvent as OpenAISessionUpdateEvent,
 )
 from pydantic import TypeAdapter
 from typing_extensions import assert_never
 from websockets.asyncio.client import ClientConnection
 
+from agents.handoffs import Handoff
+from agents.realtime._default_tracker import ModelAudioTracker
 from agents.tool import FunctionTool, Tool
 from agents.util._types import MaybeAwaitable
 
 from ..exceptions import UserError
 from ..logger import logger
+from ..version import __version__
 from .config import (
     RealtimeModelTracingConfig,
     RealtimeSessionModelSettings,
@@ -37,6 +73,8 @@ from .model import (
     RealtimeModel,
     RealtimeModelConfig,
     RealtimeModelListener,
+    RealtimePlaybackState,
+    RealtimePlaybackTracker,
 )
 from .model_events import (
     RealtimeModelAudioDoneEvent,
@@ -48,6 +86,7 @@ from .model_events import (
     RealtimeModelInputAudioTranscriptionCompletedEvent,
     RealtimeModelItemDeletedEvent,
     RealtimeModelItemUpdatedEvent,
+    RealtimeModelRawServerEvent,
     RealtimeModelToolCallEvent,
     RealtimeModelTranscriptDeltaEvent,
     RealtimeModelTurnEndedEvent,
@@ -63,6 +102,8 @@ from .model_inputs import (
     RealtimeModelSendUserInput,
 )
 
+_USER_AGENT = f"Agents/Python {__version__}"
+
 DEFAULT_MODEL_SETTINGS: RealtimeSessionModelSettings = {
     "voice": "ash",
     "modalities": ["text", "audio"],
@@ -96,11 +137,11 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
         self._websocket_task: asyncio.Task[None] | None = None
         self._listeners: list[RealtimeModelListener] = []
         self._current_item_id: str | None = None
-        self._audio_start_time: datetime | None = None
-        self._audio_length_ms: float = 0.0
+        self._audio_state_tracker: ModelAudioTracker = ModelAudioTracker()
         self._ongoing_response: bool = False
-        self._current_audio_content_index: int | None = None
         self._tracing_config: RealtimeModelTracingConfig | Literal["auto"] | None = None
+        self._playback_tracker: RealtimePlaybackTracker | None = None
+        self._created_session: OpenAISessionObject | None = None
 
     async def connect(self, options: RealtimeModelConfig) -> None:
         """Establish a connection to the model and keep it alive."""
@@ -109,6 +150,8 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
 
         model_settings: RealtimeSessionModelSettings = options.get("initial_model_settings", {})
 
+        self._playback_tracker = options.get("playback_tracker", None)
+
         self.model = model_settings.get("model_name", self.model)
         api_key = await get_api_key(options.get("api_key"))
 
@@ -126,7 +169,9 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
             "Authorization": f"Bearer {api_key}",
             "OpenAI-Beta": "realtime=v1",
         }
-        self._websocket = await websockets.connect(url, additional_headers=headers)
+        self._websocket = await websockets.connect(
+            url, user_agent_header=_USER_AGENT, additional_headers=headers
+        )
         self._websocket_task = asyncio.create_task(self._listen_for_messages())
         await self._update_session_config(model_settings)
 
@@ -135,12 +180,11 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
     ) -> None:
         """Update tracing configuration via session.update event."""
         if tracing_config is not None:
+            converted_tracing_config = _ConversionHelper.convert_tracing_config(tracing_config)
             await self._send_raw_message(
-                RealtimeModelSendRawMessage(
-                    message={
-                        "type": "session.update",
-                        "other_data": {"session": {"tracing": tracing_config}},
-                    }
+                OpenAISessionUpdateEvent(
+                    session=OpenAISessionObject(tracing=converted_tracing_config),
+                    type="session.update",
                 )
             )
 
@@ -182,7 +226,7 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
 
         except websockets.exceptions.ConnectionClosedOK:
             # Normal connection closure - no exception event needed
-            logger.info("WebSocket connection closed normally")
+            logger.debug("WebSocket connection closed normally")
         except websockets.exceptions.ConnectionClosed as e:
             await self._emit_event(
                 RealtimeModelExceptionEvent(
@@ -199,7 +243,11 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
     async def send_event(self, event: RealtimeModelSendEvent) -> None:
         """Send an event to the model."""
         if isinstance(event, RealtimeModelSendRawMessage):
-            await self._send_raw_message(event)
+            converted = _ConversionHelper.try_convert_raw_message(event)
+            if converted is not None:
+                await self._send_raw_message(converted)
+            else:
+                logger.error(f"Failed to convert raw message: {event}")
         elif isinstance(event, RealtimeModelSendUserInput):
             await self._send_user_input(event)
         elif isinstance(event, RealtimeModelSendAudio):
@@ -214,77 +262,33 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
             assert_never(event)
             raise ValueError(f"Unknown event type: {type(event)}")
 
-    async def _send_raw_message(self, event: RealtimeModelSendRawMessage) -> None:
+    async def _send_raw_message(self, event: OpenAIRealtimeClientEvent) -> None:
         """Send a raw message to the model."""
         assert self._websocket is not None, "Not connected"
 
-        converted_event = {
-            "type": event.message["type"],
-        }
-
-        converted_event.update(event.message.get("other_data", {}))
-
-        await self._websocket.send(json.dumps(converted_event))
+        await self._websocket.send(event.model_dump_json(exclude_none=True, exclude_unset=True))
 
     async def _send_user_input(self, event: RealtimeModelSendUserInput) -> None:
-        message = (
-            event.user_input
-            if isinstance(event.user_input, dict)
-            else {
-                "type": "message",
-                "role": "user",
-                "content": [{"type": "input_text", "text": event.user_input}],
-            }
-        )
-        other_data = {
-            "item": message,
-        }
-
-        await self._send_raw_message(
-            RealtimeModelSendRawMessage(
-                message={"type": "conversation.item.create", "other_data": other_data}
-            )
-        )
-        await self._send_raw_message(
-            RealtimeModelSendRawMessage(message={"type": "response.create"})
-        )
+        converted = _ConversionHelper.convert_user_input_to_item_create(event)
+        await self._send_raw_message(converted)
+        await self._send_raw_message(OpenAIResponseCreateEvent(type="response.create"))
 
     async def _send_audio(self, event: RealtimeModelSendAudio) -> None:
-        base64_audio = base64.b64encode(event.audio).decode("utf-8")
-        await self._send_raw_message(
-            RealtimeModelSendRawMessage(
-                message={
-                    "type": "input_audio_buffer.append",
-                    "other_data": {
-                        "audio": base64_audio,
-                    },
-                }
-            )
-        )
+        converted = _ConversionHelper.convert_audio_to_input_audio_buffer_append(event)
+        await self._send_raw_message(converted)
         if event.commit:
             await self._send_raw_message(
-                RealtimeModelSendRawMessage(message={"type": "input_audio_buffer.commit"})
+                OpenAIInputAudioBufferCommitEvent(type="input_audio_buffer.commit")
             )
 
     async def _send_tool_output(self, event: RealtimeModelSendToolOutput) -> None:
-        await self._send_raw_message(
-            RealtimeModelSendRawMessage(
-                message={
-                    "type": "conversation.item.create",
-                    "other_data": {
-                        "item": {
-                            "type": "function_call_output",
-                            "output": event.output,
-                            "call_id": event.tool_call.id,
-                        },
-                    },
-                }
-            )
-        )
+        converted = _ConversionHelper.convert_tool_output(event)
+        await self._send_raw_message(converted)
 
         tool_item = RealtimeToolCallItem(
             item_id=event.tool_call.id or "",
             previous_item_id=event.tool_call.previous_item_id,
+            call_id=event.tool_call.call_id,
             type="function_call",
             status="completed",
             arguments=event.tool_call.arguments,
@@ -294,36 +298,78 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
         await self._emit_event(RealtimeModelItemUpdatedEvent(item=tool_item))
 
         if event.start_response:
-            await self._send_raw_message(
-                RealtimeModelSendRawMessage(message={"type": "response.create"})
-            )
+            await self._send_raw_message(OpenAIResponseCreateEvent(type="response.create"))
+
+    def _get_playback_state(self) -> RealtimePlaybackState:
+        if self._playback_tracker:
+            return self._playback_tracker.get_state()
+
+        if last_audio_item_id := self._audio_state_tracker.get_last_audio_item():
+            item_id, item_content_index = last_audio_item_id
+            audio_state = self._audio_state_tracker.get_state(item_id, item_content_index)
+            if audio_state:
+                elapsed_ms = (
+                    datetime.now() - audio_state.initial_received_time
+                ).total_seconds() * 1000
+                return {
+                    "current_item_id": item_id,
+                    "current_item_content_index": item_content_index,
+                    "elapsed_ms": elapsed_ms,
+                }
+
+        return {
+            "current_item_id": None,
+            "current_item_content_index": None,
+            "elapsed_ms": None,
+        }
 
     async def _send_interrupt(self, event: RealtimeModelSendInterrupt) -> None:
-        if not self._current_item_id or not self._audio_start_time:
+        playback_state = self._get_playback_state()
+        current_item_id = playback_state.get("current_item_id")
+        current_item_content_index = playback_state.get("current_item_content_index")
+        elapsed_ms = playback_state.get("elapsed_ms")
+        if current_item_id is None or elapsed_ms is None:
+            logger.debug(
+                "Skipping interrupt. "
+                f"Item id: {current_item_id}, "
+                f"elapsed ms: {elapsed_ms}, "
+                f"content index: {current_item_content_index}"
+            )
             return
 
-        await self._cancel_response()
-
-        elapsed_time_ms = (datetime.now() - self._audio_start_time).total_seconds() * 1000
-        if elapsed_time_ms > 0 and elapsed_time_ms < self._audio_length_ms:
-            await self._emit_event(RealtimeModelAudioInterruptedEvent())
-            await self._send_raw_message(
-                RealtimeModelSendRawMessage(
-                    message={
-                        "type": "conversation.item.truncate",
-                        "other_data": {
-                            "item_id": self._current_item_id,
-                            "content_index": self._current_audio_content_index,
-                            "audio_end_ms": elapsed_time_ms,
-                        },
-                    }
+        current_item_content_index = current_item_content_index or 0
+        if elapsed_ms > 0:
+            await self._emit_event(
+                RealtimeModelAudioInterruptedEvent(
+                    item_id=current_item_id,
+                    content_index=current_item_content_index,
                 )
             )
+            converted = _ConversionHelper.convert_interrupt(
+                current_item_id,
+                current_item_content_index,
+                int(elapsed_ms),
+            )
+            await self._send_raw_message(converted)
+        else:
+            logger.debug(
+                "Didn't interrupt bc elapsed ms is < 0. "
+                f"Item id: {current_item_id}, "
+                f"elapsed ms: {elapsed_ms}, "
+                f"content index: {current_item_content_index}"
+            )
 
-        self._current_item_id = None
-        self._audio_start_time = None
-        self._audio_length_ms = 0.0
-        self._current_audio_content_index = None
+        automatic_response_cancellation_enabled = (
+            self._created_session
+            and self._created_session.turn_detection
+            and self._created_session.turn_detection.interrupt_response
+        )
+        if not automatic_response_cancellation_enabled:
+            await self._cancel_response()
+
+        self._audio_state_tracker.on_interrupted()
+        if self._playback_tracker:
+            self._playback_tracker.on_interrupted()
 
     async def _send_session_update(self, event: RealtimeModelSendSessionUpdate) -> None:
         """Send a session update to the model."""
@@ -331,22 +377,20 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
 
     async def _handle_audio_delta(self, parsed: ResponseAudioDeltaEvent) -> None:
         """Handle audio delta events and update audio tracking state."""
-        self._current_audio_content_index = parsed.content_index
         self._current_item_id = parsed.item_id
-        if self._audio_start_time is None:
-            self._audio_start_time = datetime.now()
-            self._audio_length_ms = 0.0
 
         audio_bytes = base64.b64decode(parsed.delta)
-        # Calculate audio length in ms using 24KHz pcm16le
-        self._audio_length_ms += self._calculate_audio_length_ms(audio_bytes)
-        await self._emit_event(
-            RealtimeModelAudioEvent(data=audio_bytes, response_id=parsed.response_id)
-        )
 
-    def _calculate_audio_length_ms(self, audio_bytes: bytes) -> float:
-        """Calculate audio length in milliseconds for 24KHz PCM16LE format."""
-        return len(audio_bytes) / 24 / 2
+        self._audio_state_tracker.on_audio_delta(parsed.item_id, parsed.content_index, audio_bytes)
+
+        await self._emit_event(
+            RealtimeModelAudioEvent(
+                data=audio_bytes,
+                response_id=parsed.response_id,
+                item_id=parsed.item_id,
+                content_index=parsed.content_index,
+            )
+        )
 
     async def _handle_output_item(self, item: ConversationItem) -> None:
         """Handle response output item events (function calls and messages)."""
@@ -354,6 +398,7 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
             tool_call = RealtimeToolCallItem(
                 item_id=item.id or "",
                 previous_item_id=None,
+                call_id=item.call_id,
                 type="function_call",
                 # We use the same item for tool call and output, so it will be completed by the
                 # output being added
@@ -365,7 +410,7 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
             await self._emit_event(RealtimeModelItemUpdatedEvent(item=tool_call))
             await self._emit_event(
                 RealtimeModelToolCallEvent(
-                    call_id=item.id or "",
+                    call_id=item.call_id or "",
                     name=item.name or "",
                     arguments=item.arguments or "",
                     id=item.id or "",
@@ -378,7 +423,9 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
                     "item_id": item.id or "",
                     "type": item.type,
                     "role": item.role,
-                    "content": item.content,
+                    "content": (
+                        [content.model_dump() for content in item.content] if item.content else []
+                    ),
                     "status": "in_progress",
                 }
             )
@@ -404,12 +451,11 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
 
     async def _cancel_response(self) -> None:
         if self._ongoing_response:
-            await self._send_raw_message(
-                RealtimeModelSendRawMessage(message={"type": "response.cancel"})
-            )
+            await self._send_raw_message(OpenAIResponseCancelEvent(type="response.cancel"))
             self._ongoing_response = False
 
     async def _handle_ws_event(self, event: dict[str, Any]):
+        await self._emit_event(RealtimeModelRawServerEvent(data=event))
         try:
             if "previous_item_id" in event and event["previous_item_id"] is None:
                 event["previous_item_id"] = ""  # TODO (rm) remove
@@ -438,7 +484,12 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
         if parsed.type == "response.audio.delta":
             await self._handle_audio_delta(parsed)
         elif parsed.type == "response.audio.done":
-            await self._emit_event(RealtimeModelAudioDoneEvent())
+            await self._emit_event(
+                RealtimeModelAudioDoneEvent(
+                    item_id=parsed.item_id,
+                    content_index=parsed.content_index,
+                )
+            )
         elif parsed.type == "input_audio_buffer.speech_started":
             await self._send_interrupt(RealtimeModelSendInterrupt())
         elif parsed.type == "response.created":
@@ -449,6 +500,9 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
             await self._emit_event(RealtimeModelTurnEndedEvent())
         elif parsed.type == "session.created":
             await self._send_tracing_config(self._tracing_config)
+            self._update_created_session(parsed.session)  # type: ignore
+        elif parsed.type == "session.updated":
+            self._update_created_session(parsed.session)  # type: ignore
         elif parsed.type == "error":
             await self._emit_event(RealtimeModelErrorEvent(error=parsed.error))
         elif parsed.type == "conversation.item.deleted":
@@ -466,16 +520,13 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
             parsed.type == "conversation.item.input_audio_transcription.completed"
             or parsed.type == "conversation.item.truncated"
         ):
-            await self._send_raw_message(
-                RealtimeModelSendRawMessage(
-                    message={
-                        "type": "conversation.item.retrieve",
-                        "other_data": {
-                            "item_id": self._current_item_id,
-                        },
-                    }
+            if self._current_item_id:
+                await self._send_raw_message(
+                    OpenAIConversationItemRetrieveEvent(
+                        type="conversation.item.retrieve",
+                        item_id=self._current_item_id,
+                    )
                 )
-            )
             if parsed.type == "conversation.item.input_audio_transcription.completed":
                 await self._emit_event(
                     RealtimeModelInputAudioTranscriptionCompletedEvent(
@@ -501,17 +552,17 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
         ):
             await self._handle_output_item(parsed.item)
 
+    def _update_created_session(self, session: OpenAISessionObject) -> None:
+        self._created_session = session
+        if session.output_audio_format:
+            self._audio_state_tracker.set_audio_format(session.output_audio_format)
+            if self._playback_tracker:
+                self._playback_tracker.set_audio_format(session.output_audio_format)
+
     async def _update_session_config(self, model_settings: RealtimeSessionModelSettings) -> None:
         session_config = self._get_session_config(model_settings)
         await self._send_raw_message(
-            RealtimeModelSendRawMessage(
-                message={
-                    "type": "session.update",
-                    "other_data": {
-                        "session": session_config.model_dump(exclude_unset=True, exclude_none=True)
-                    },
-                }
-            )
+            OpenAISessionUpdateEvent(session=session_config, type="session.update")
         )
 
     def _get_session_config(
@@ -525,6 +576,7 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
                 or DEFAULT_MODEL_SETTINGS.get("model_name")
             ),
             voice=model_settings.get("voice", DEFAULT_MODEL_SETTINGS.get("voice")),
+            speed=model_settings.get("speed", None),
             modalities=model_settings.get("modalities", DEFAULT_MODEL_SETTINGS.get("modalities")),
             input_audio_format=model_settings.get(
                 "input_audio_format",
@@ -546,10 +598,14 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
                 "tool_choice",
                 DEFAULT_MODEL_SETTINGS.get("tool_choice"),  # type: ignore
             ),
-            tools=self._tools_to_session_tools(model_settings.get("tools", [])),
+            tools=self._tools_to_session_tools(
+                tools=model_settings.get("tools", []), handoffs=model_settings.get("handoffs", [])
+            ),
         )
 
-    def _tools_to_session_tools(self, tools: list[Tool]) -> list[OpenAISessionTool]:
+    def _tools_to_session_tools(
+        self, tools: list[Tool], handoffs: list[Handoff]
+    ) -> list[OpenAISessionTool]:
         converted_tools: list[OpenAISessionTool] = []
         for tool in tools:
             if not isinstance(tool, FunctionTool):
@@ -562,6 +618,17 @@ class OpenAIRealtimeWebSocketModel(RealtimeModel):
                     type="function",
                 )
             )
+
+        for handoff in handoffs:
+            converted_tools.append(
+                OpenAISessionTool(
+                    name=handoff.tool_name,
+                    description=handoff.tool_description,
+                    parameters=handoff.input_json_schema,
+                    type="function",
+                )
+            )
+
         return converted_tools
 
 
@@ -582,3 +649,98 @@ class _ConversionHelper:
                 "status": "in_progress",
             },
         )
+
+    @classmethod
+    def try_convert_raw_message(
+        cls, message: RealtimeModelSendRawMessage
+    ) -> OpenAIRealtimeClientEvent | None:
+        try:
+            data = {}
+            data["type"] = message.message["type"]
+            data.update(message.message.get("other_data", {}))
+            return TypeAdapter(OpenAIRealtimeClientEvent).validate_python(data)
+        except Exception:
+            return None
+
+    @classmethod
+    def convert_tracing_config(
+        cls, tracing_config: RealtimeModelTracingConfig | Literal["auto"] | None
+    ) -> OpenAISessionTracing | None:
+        if tracing_config is None:
+            return None
+        elif tracing_config == "auto":
+            return "auto"
+        return OpenAISessionTracingConfiguration(
+            group_id=tracing_config.get("group_id"),
+            metadata=tracing_config.get("metadata"),
+            workflow_name=tracing_config.get("workflow_name"),
+        )
+
+    @classmethod
+    def convert_user_input_to_conversation_item(
+        cls, event: RealtimeModelSendUserInput
+    ) -> OpenAIConversationItem:
+        user_input = event.user_input
+
+        if isinstance(user_input, dict):
+            return OpenAIConversationItem(
+                type="message",
+                role="user",
+                content=[
+                    OpenAIConversationItemContent(
+                        type="input_text",
+                        text=item.get("text"),
+                    )
+                    for item in user_input.get("content", [])
+                ],
+            )
+        else:
+            return OpenAIConversationItem(
+                type="message",
+                role="user",
+                content=[OpenAIConversationItemContent(type="input_text", text=user_input)],
+            )
+
+    @classmethod
+    def convert_user_input_to_item_create(
+        cls, event: RealtimeModelSendUserInput
+    ) -> OpenAIRealtimeClientEvent:
+        return OpenAIConversationItemCreateEvent(
+            type="conversation.item.create",
+            item=cls.convert_user_input_to_conversation_item(event),
+        )
+
+    @classmethod
+    def convert_audio_to_input_audio_buffer_append(
+        cls, event: RealtimeModelSendAudio
+    ) -> OpenAIRealtimeClientEvent:
+        base64_audio = base64.b64encode(event.audio).decode("utf-8")
+        return OpenAIInputAudioBufferAppendEvent(
+            type="input_audio_buffer.append",
+            audio=base64_audio,
+        )
+
+    @classmethod
+    def convert_tool_output(cls, event: RealtimeModelSendToolOutput) -> OpenAIRealtimeClientEvent:
+        return OpenAIConversationItemCreateEvent(
+            type="conversation.item.create",
+            item=OpenAIConversationItem(
+                type="function_call_output",
+                output=event.output,
+                call_id=event.tool_call.call_id,
+            ),
+        )
+
+    @classmethod
+    def convert_interrupt(
+        cls,
+        current_item_id: str,
+        current_audio_content_index: int,
+        elapsed_time_ms: int,
+    ) -> OpenAIRealtimeClientEvent:
+        return OpenAIConversationItemTruncateEvent(
+            type="conversation.item.truncate",
+            item_id=current_item_id,
+            content_index=current_audio_content_index,
+            audio_end_ms=elapsed_time_ms,
+        )
